unit demoForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ps40import;

type
  TForm1 = class(TForm)
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    GroupBox2: TGroupBox;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    Label19: TLabel;
    Label20: TLabel;
    Label21: TLabel;
    Label22: TLabel;
    Label23: TLabel;
    Label24: TLabel;
    Label25: TLabel;
    Label26: TLabel;
    Button1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    FOptions  : TSharewareOptions;
    FShareInfo: TSharewareInfo;
    FUserInfo : TUserInfo;
    FShareOK  : Boolean;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses demoRegDlg;

{$R *.DFM}

const
  BoolStrings       : Array[Boolean] of String = ('False','True');

{ Convert the given Registry key to string }
function KeyToString(AKey: HKEY): String;
begin
  Result := 'HKEY_CLASSES_ROOT';
  case AKey of
    HKEY_CURRENT_USER   : Result := 'HKEY_CURRENT_USER';
    HKEY_LOCAL_MACHINE  : Result := 'HKEY_LOCAL_MACHINE';
    HKEY_USERS          : Result := 'HKEY_USERS';
    HKEY_CURRENT_CONFIG : Result := 'HKEY_CURRENT_CONFIG';
    HKEY_DYN_DATA       : Result := 'HKEY_DYN_DATA';
  end;
end;

{ Convert the given expiration mode to string }
function ModeToString(AMode: Integer): String;
begin
  Result := 'EM_DONTEXPIRE';
  case AMode of
    EM_TRIALPERIOD     : Result := 'EM_TRIALPERIOD';
    EM_RUNCOUNT        : Result := 'EM_RUNCOUNT';
    EM_COMBINED        : Result := 'EM_COMBINED';
    EM_EXPIRATIONDATE  : Result := 'EM_EXPIRATIONDATE';
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  Result: DWORD;
  s     : String;
begin
  {
    Initialize shareware options
    ==============================

    You need to do this before you access the PerfectShareware Library. The best place to do it is the
    FormCreate event handler, because you will probably call ReadSharewareInformation from here. Having
    initialized this record once, it is recommended not to change it later, because this might result in
    wrong information being written to the Registry, thus, the shareware mechanism not working correctly.
  }
  with FOptions do begin
    soAppID               := 'ShareDemo';
    soEnableCheckDefeat   := True;
    soEnableSaveOnDefeat  := False;
    soRegRoot             := HKEY_CURRENT_USER;
    soRegBackupRoot       := HKEY_LOCAL_MACHINE;
    soExpirationMode      := EM_TRIALPERIOD;
    soExpirationDate      := Trunc(Now);
    soGracePeriod         := 34;
    soTrialPeriod         := 30;
    soMaxNumRuns          := 20;
    soState               := 0;
    StrCopy(soRegKey,'Software\Microsoft\Windows\CurrentVersion\Explorer');
    StrCopy(soRegBackupKey,'Software\Microsoft\Windows\CurrentVersion');
  end;

  {
    Try to read the information
    ===========================

    This is the second step when turning your application into shareware. The PerfectShareware Library will
    try to read the shareware information from the Registry and return the current shareware state.

    PLEASE NOTE: Please refrain from modifying FOptions.soState after calling ReadSharewareInformation. This
                 flag will later on be used by WriteSharewareInformation. If you modify this flag, wrong
                 information might be written to the Registry, this, the shareware mechanism might not work
                 correctly!
  }
  Result   := ReadSharewareInformation(FOptions,FShareInfo,FUserInfo);

  {
    Evaluate the result code
    ========================

    This is also important! Result codes other than PSERR_SUCCESS either indicate wrong or missing parameters
    in FOptions, or errors that occurred while accessing the Registry. You should handle these errors and
    abort the application.

    PLEASE NOTE: While the shareware state is stored in FOptions and reused later, the Result code is not
                 stored. You should store it yourself and make sure you don't write the information to the
                 registry without having properly read it here!
  }
  if Result <> PSERR_SUCCESS then begin
    FShareOK := False;
    MessageDlg('An error occurred while accessing the Registry! The application will be terminated. Please '+
               'restart Windows before trying to run the application again.',mtError,[mbOK],0);
    Application.Terminate;
    exit;
  end else
    FShareOK := True;

  {
    Evaluate status code
    ====================

    This step does the real shareware functionality. Here, the status code must be evaluated. This code is
    created by the PerfectShareware Library according to the options you set in FOptions.
  }
  if (FOptions.soState and SF_INITIALRUN) = SF_INITIALRUN then begin
    { This indicates that your application is running for the first time. You can display a welcome message
      here. }
    MessageDlg('Thank you for using MyApplication. This is shareware, so please register! I''ll give you '+
               '30 days...',mtInformation,[mbOK],0);
  end else

  if (FOptions.soState and SF_DEFEATED) = SF_DEFEATED then begin
    { One of the registration sets was (partly) removed. You should act properly in this case! }
    MessageDlg('Well, you may not try to defeat the shareware mechanism! Bye...',mtWarning,[mbOK],0);
    Application.Terminate;
    exit;
  end else

  if (FOptions.soState and SF_DATEPATCHED) = SF_DATEPATCHED then begin
    { The user turned back the clock. You should act properly in this case! }
    MessageDlg('Are you sure your computer is displaying the correct time?? Bye...',mtWarning,[mbOK],0);
    Application.Terminate;
    exit;
  end else

  if (FOptions.soState and SF_INFOPATCHED) = SF_INFOPATCHED then begin
    { The user modified the registry information. You should act properly in this case! }
    MessageDlg('Stop messing up the Registry! Bye...',mtWarning,[mbOK],0);
    Application.Terminate;
    exit;
  end else

  if (FOptions.soState and SF_EXPIRED) = SF_EXPIRED then begin
    { The application expired. }
    MessageDlg('I told you I''d give you 30 days to register...',mtWarning,[mbOK],0);
    Application.Terminate;
    exit;
  end else

  if (FOptions.soState and SF_NOSTATE) = SF_NOSTATE then begin
    { This is the "normal" state. You can check for one more flag here }
    if (FOptions.soState and SF_OVERTRIALPERIOD) = SF_OVERTRIALPERIOD then begin
      s := 'Hurry up!'#13#13#10;
      s := s+Format('Day %d of your grace period.',[Trunc(now)-Trunc(FShareInfo.siInitialRun)-FOptions.soTrialPeriod+1]);
      MessageDlg(s,mtInformation,[mbOK],0);
    end else if (FOptions.soState and SF_REGISTERED) <> SF_REGISTERED then begin
      s := 'Still evaluating?'#13#13#10;
      s := s+Format('The application was run %d time(s).'#13#10,[FShareInfo.siRunCount]);
      s := s+Format('Day %d of your trial period.',[Trunc(now)-Trunc(FShareInfo.siInitialRun)+1]);
      MessageDlg(s,mtInformation,[mbOK],0);
    end;
  end;

  Label1.Caption := GetDLLName;
  Label2.Caption := GetDLLVersion;
  Label4.Caption := FOptions.soAppID;
  Label6.Caption := BoolStrings[FOptions.soEnableCheckDefeat];
  Label8.Caption := BoolStrings[FOptions.soEnableSaveOnDefeat];
  Label10.Caption := KeyToString(FOptions.soRegRoot);
  Label12.Caption := FOptions.soRegKey;
  Label14.Caption := KeyToString(FOptions.soRegBackupRoot);
  Label16.Caption := FOptions.soRegBackupKey;
  Label18.Caption := ModeToString(FOptions.soExpirationMode);
  Label20.Caption := DateToStr(FOptions.soExpirationDate);
  Label22.Caption := IntToStr(FOptions.soTrialPeriod);
  Label24.Caption := IntToStr(FOptions.soGracePeriod);
  Label26.Caption := IntToStr(FOptions.soMaxNumRuns);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  {
    Try to store the information
    ============================

    Finally, you must store the information. You need to make sure not to store the information when
    it couldn't be properly read before. That's why the FShareOK flag was declared. It is True when
    there was no error accessing the Registry.

    PLEASE NOTE: All values are updated by the PerfectShareware Library. If you modify any value, the
                 shareware information will be incorrect!
  }
  if FShareOK then WriteSharewareInformation(FOptions,FShareInfo,FUserInfo);
end;

procedure TForm1.Button1Click(Sender: TObject);
var
  UName, UCode: String;
begin
  if RegDlg.ShowModal = mrOK then begin
    UName := Trim(RegDlg.Edit1.Text);
    UCode := Trim(RegDlg.Edit2.Text);

    if RegisterUser(UName,StrPas(FOptions.soAppID),'',UCode,FShareInfo,FUserInfo) <> PSERR_SUCCESS then
      MessageDlg('Registration failed! Please check your data and try again!',mtError,[mbOK],0)
  end;
end;

end.
