{ ****************************************************************************************************************************** }
{ ps40.pas                                                                      Copyright  1997-99 DithoSoft Software Solutions }
{ Version 4.0                                                                                                All rights reserved }
{ ------------------------------------------------------------------------------------------------------------------------------ }
{ This unit contains the wrapper component for PS40.DLL. For declarations of additional types and constants, please refer to     }
{ the ps40import.pas unit.                                                                                                       }
{ ------------------------------------------------------------------------------------------------------------------------------ }
{ The source codes of the component are provided for reference purposes only. You may not derive new components from this one.   }
{ You may also not create an entirely new product with the same or similar functionality based on this source code.              }
{ ****************************************************************************************************************************** }
unit ps40;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, ps40Import;

type
  { EPerfectSharewareError }
  EPerfectSharewareError = class(Exception)
  private
    FErrorCode : DWORD;
  public
    constructor CreateCode(AErrCode: DWORD);
  public
    property ErrorCode: DWORD read FErrorCode;
  end;

  { TExpirationMode }
  TExpirationMode = (emDontExpire, emTrialPeriod, emRunCount, emCombined, emExpirationDate);

  { TRegRootKey }
  TRegRootKey = (hkClassesRoot, hkCurrentUser, hkLocalMachine, hkUsers, hkCurrentConfig, hkDynData);

  { TEvaluationPeriod }
  TEvaluationPeriod = 1..MAXINT;

  { TRunCount }
  TRunCount = 1..MAXINT;

  { TPerfectShareware }
  TPerfectShareware = class(TComponent)
  private
    { Property/Event variables }
    FAutoActive      : Boolean;
    FSharewareInfo   : TSharewareInfo;
    FSharewareOK     : Boolean;
    FSharewareOptions: TSharewareOptions;
    FUserInfo        : TUserInfo;
    FOnDatePatched   : TNotifyEvent;
    FOnDefeated      : TNotifyEvent;
    FOnExpired       : TNotifyEvent;
    FOnInfoPatched   : TNotifyEvent;
    FOnInitialRun    : TNotifyEvent;
    FOnNagScreen     : TNotifyEvent;
    FOnRefresh       : TNotifyEvent;

    { These are for setting published properties }
    procedure   SetAppID(Value: String);
    procedure   SetAutoActive(Value: Boolean);
    procedure   SetEnableCheckDefeat(Value: Boolean);
    procedure   SetEnableSaveOnDefeat(Value: Boolean);
    procedure   SetExpirationDate(Value: TDateTime);
    procedure   SetExpirationMode(Value: TExpirationMode);
    procedure   SetGracePeriod(Value: TEvaluationPeriod);
    procedure   SetMaxNumRuns(Value: TRunCount);
    procedure   SetRegBackupKey(Value: String);
    procedure   SetRegBackupRoot(Value: TRegRootKey);
    procedure   SetRegKey(Value: String);
    procedure   SetRegRoot(Value: TRegRootKey);
    procedure   SetTrialPeriod(Value: TEvaluationPeriod);

    { These are for retrieving published properties }
    function    GetAppID: String;
    function    GetEnableCheckDefeat: Boolean;
    function    GetEnableSaveOnDefeat: Boolean;
    function    GetExpirationDate: TDateTime;
    function    GetExpirationMode: TExpirationMode;
    function    GetGracePeriod: TEvaluationPeriod;
    function    GetMaxNumRuns: TRunCount;
    function    GetRegBackupKey: String;
    function    GetRegBackupRoot: TRegRootKey;
    function    GetRegKey: String;
    function    GetRegRoot: TRegRootKey;
    function    GetTrialPeriod: TEvaluationPeriod;

    { These are for setting public properties }
    procedure   SetInitialRun(Value: TDateTime);
    procedure   SetLastRun(Value: TDateTime);
    procedure   SetNumRuns(Value: TRunCount);
    procedure   SetRegisteredUser(Value: String);
    procedure   SetRegistrationCode(Value: String);

    { These are for retrieving public properties }
    function    GetIsInitialRun: Boolean;
    function    GetIsInfoPatched: Boolean;
    function    GetIsDatePatched: Boolean;
    function    GetIsDefeated: Boolean;
    function    GetIsExpired: Boolean;
    function    GetIsRegistered: Boolean;
    function    GetInitialRun: TDateTime;
    function    GetLastRun: TDateTime;
    function    GetNumRuns: TRunCount;
    function    GetRemainingTrialPeriod: Integer;
    function    GetRemainingGracePeriod: Integer;
    function    GetRemainingRuns: Integer;
    function    GetRegisteredUser: String;
    function    GetRegistrationCode: String;

    { These are internal tools }
    function    RootKeyToHKey(AKey: TRegRootKey): HKEY;
    function    HKeyToRootKey(AKey: HKEY): TRegRootKey;
    procedure   NotifyInitialRun;
    procedure   NotifyInfoPatched;
    procedure   NotifyDatePatched;
    procedure   NotifyDefeated;
    procedure   NotifyExpired;
    procedure   NotifyNagScreen;
    procedure   NotifyRefresh;
  protected
    { Overridden protected methods }
    procedure   Loaded; override;
  public
    { Constructor/Destructor }
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;

    { Public interface methods }
    procedure   LoadSecretInfo;
    procedure   StoreSecretInfo;
    procedure   ClearSecretInfo;
    procedure   RefreshSecretInfo;
    function    UserRegistration(const AName, ACode, APluginData: String): Boolean;
  public
    { Public read-only properties }
    property    SharewareOK          : Boolean           read FSharewareOK;
    property    IsInitialRun         : Boolean           read GetIsInitialRun;
    property    IsInfoPatched        : Boolean           read GetIsInfoPatched;
    property    IsDatePatched        : Boolean           read GetIsDatePatched;
    property    IsDefeated           : Boolean           read GetIsDefeated;
    property    IsExpired            : Boolean           read GetIsExpired;
    property    IsRegistered         : Boolean           read GetIsRegistered;
    property    InitialRun           : TDateTime         read GetInitialRun            write SetInitialRun;
    property    LastRun              : TDateTime         read GetLastRun               write SetLastRun;
    property    NumRuns              : TRunCount         read GetNumRuns               write SetNumRuns;
    property    RemainingTrialPeriod : Integer           read GetRemainingTrialPeriod;
    property    RemainingGracePeriod : Integer           read GetRemainingGracePeriod;
    property    RemainingRuns        : Integer           read GetRemainingRuns;
    property    RegisteredUser       : String            read GetRegisteredUser        write SetRegisteredUser;
    property    RegistrationCode     : String            read GetRegistrationCode      write SetRegistrationCode;
  published
    { Published properties }
    property    AppID                : String            read GetAppID                 write SetAppID;
    property    AutoActive           : Boolean           read FAutoActive              write SetAutoActive;
    property    EnableCheckDefeat    : Boolean           read GetEnableCheckDefeat     write SetEnableCheckDefeat;
    property    EnableSaveOnDefeat   : Boolean           read GetEnableSaveOnDefeat    write SetEnableSaveOnDefeat;
    property    ExpirationMode       : TExpirationMode   read GetExpirationMode        write SetExpirationMode;
    property    ExpirationDate       : TDateTime         read GetExpirationDate        write SetExpirationDate;
    property    GracePeriod          : TEvaluationPeriod read GetGracePeriod           write SetGracePeriod;
    property    MaxNumRuns           : TRunCount         read GetMaxNumRuns            write SetMaxNumRuns;
    property    RegBackupKey         : String            read GetRegBackupKey          write SetRegBackupKey;
    property    RegBackupRoot        : TRegRootKey       read GetRegBackupRoot         write SetRegBackupRoot;
    property    RegKey               : String            read GetRegKey                write SetRegKey;
    property    RegRoot              : TRegRootKey       read GetRegRoot               write SetRegRoot;
    property    TrialPeriod          : TEvaluationPeriod read GetTrialPeriod           write SetTrialPeriod;

    { Published events }
    property    OnInitialRun         : TNotifyEvent      read FOnInitialRun            write FOnInitialRun;
    property    OnInfoPatched        : TNotifyEvent      read FOnInfoPatched           write FOnInfoPatched;
    property    OnDatePatched        : TNotifyEvent      read FOnDatePatched           write FOnDatePatched;
    property    OnDefeated           : TNotifyEvent      read FOnDefeated              write FOnDefeated;
    property    OnExpired            : TNotifyEvent      read FOnExpired               write FOnExpired;
    property    OnNagScreen          : TNotifyEvent      read FOnNagScreen             write FOnNagScreen;
    property    OnRefresh            : TNotifyEvent      read FOnRefresh               write FOnRefresh;
  end;

procedure Register;

implementation

uses ps40PropEdits;

resourcestring
  SInvalidValueLength = 'The length of this value must be between %d and %d!';


// Component/Property editor registration ////////////////////////////////////////////////////////////////////////////////////////
procedure Register;
begin
  { Register component and property editors }
  RegisterComponents('Shareware', [TPerfectShareware]);
  RegisterPropertyEditors;
end;

// PerfectShareware exception class //////////////////////////////////////////////////////////////////////////////////////////////
constructor EPerfectSharewareError.CreateCode(AErrCode: DWORD);
begin
  { Initialize ErrorCode property }
  FErrorCode := AErrCode;
end;


// Component constructor/destructor //////////////////////////////////////////////////////////////////////////////////////////////
constructor TPerfectShareware.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  { Initialize properties }
  FAutoActive            := True;
  FSharewareOK           := True;
  with FSharewareOptions do begin
    soAppID              := 'MyApplication';
    soEnableCheckDefeat  := True;
    soEnableSaveOnDefeat := False;
    soRegRoot            := HKEY_CURRENT_USER;
    soRegBackupRoot      := HKEY_LOCAL_MACHINE;
    soRegKey             := 'Software\Microsoft\Windows\CurrentVersion\Explorer';
    soRegBackupKey       := 'Software\Microsoft\Windows\CurrentVersion';
    soExpirationMode     := ord(emTrialPeriod);
    soExpirationDate     := Trunc(now)+30;
    soGracePeriod        := 34;
    soTrialPeriod        := 30;
    soMaxNumRuns         := 50;
  end;
  FillChar(FSharewareInfo,SizeOf(TSharewareInfo),0);
  FillChar(FUserInfo,SizeOf(TUserInfo),0);
end;

destructor TPerfectShareware.Destroy;
begin
  { When destroying and auto active, store info }
  if FAutoActive then StoreSecretInfo;
  inherited Destroy;
end;


// Protected overridden methods //////////////////////////////////////////////////////////////////////////////////////////////////
procedure TPerfectShareware.Loaded;
begin
  { When loading and auto active (NOT in design mode), load info }
  inherited Loaded;
  if FAutoActive then LoadSecretInfo;
end;


// Public interface methods //////////////////////////////////////////////////////////////////////////////////////////////////////
procedure TPerfectShareware.LoadSecretInfo;
var
  Result: DWORD;
begin
  { Do not load the secret info when in design mode }
  if (csDesigning in ComponentState) then exit;

  try
    { Try to read the information from the Registry }
    Result := ReadSharewareInformation(FSharewareOptions,FSharewareInfo,FUserInfo);

    { Check the result and raise exception if an error occurred }
    if Result <> PSERR_SUCCESS then begin
      FSharewareOK := False;
      raise EPerfectSharewareError.CreateCode(Result);
    end;

    { Evaluate the status code }
    if (FSharewareOptions.soState and SF_INITIALRUN) = SF_INITIALRUN then
      NotifyInitialRun
    else if (FSharewareOptions.soState and SF_DEFEATED) = SF_DEFEATED then
      NotifyDefeated
    else if (FSharewareOptions.soState and SF_DATEPATCHED) = SF_DATEPATCHED then
      NotifyDatePatched
    else if (FSharewareOptions.soState and SF_INFOPATCHED) = SF_INFOPATCHED then
      NotifyInfoPatched
    else if (FSharewareOptions.soState and SF_EXPIRED) = SF_EXPIRED then
      NotifyExpired
    else if (FSharewareOptions.soState and SF_NOSTATE) = SF_NOSTATE then begin
      if (FSharewareOptions.soState and SF_REGISTERED) <> SF_REGISTERED then NotifyNagScreen;
    end;

  except
    { Hand on exceptions }
    raise;
  end;
end;

procedure TPerfectShareware.StoreSecretInfo;
var
  Result: DWORD;
begin
  { Do not store the secret info when in design mode }
  if (csDesigning in ComponentState) then exit;

  { Store the Registry information if it was read properly in Loaded method }
  try
    if FSharewareOK then begin
      Result := WriteSharewareInformation(FSharewareOptions,FSharewareInfo,FUserInfo);

      { Check the result and raise exception if an error occurred }
      if (Result <> PSERR_SUCCESS) and (Result <> PSERR_SAVEDENIED) then begin
        FSharewareOK := False;
        raise EPerfectSharewareError.CreateCode(Result);
      end;
    end;
  except
    raise;
  end;
end;

procedure TPerfectShareware.ClearSecretInfo;
begin
  { Clear the shareware information from the Registry }
  RemoveSharewareInformation(FSharewareOptions);
end;

procedure TPerfectShareware.RefreshSecretInfo;
var
  Result: DWORD;
  so    : TSharewareOptions;
  si    : TSharewareInfo;
  ui    : TUserInfo;
begin
  { Do not refresh when in design mode }
  if (csDesigning in ComponentState) then exit;

  { Notify now when registered }
  if FSharewareInfo.siRegistered then begin
    NotifyRefresh;
    exit;
  end;

  { Copy options }
  with so do begin
    soAppID              := FSharewareOptions.soAppID;
    soEnableCheckDefeat  := FSharewareOptions.soEnableCheckDefeat;
    soEnableSaveOnDefeat := FSharewareOptions.soEnableSaveOnDefeat;
    soRegRoot            := FSharewareOptions.soRegRoot;
    soRegBackupRoot      := FSharewareOptions.soRegBackupRoot;
    soRegKey             := FSharewareOptions.soRegKey;
    soRegBackupKey       := FSharewareOptions.soRegBackupKey;
    soExpirationMode     := FSharewareOptions.soExpirationMode;
    soExpirationDate     := FSharewareOptions.soExpirationDate;
    soGracePeriod        := FSharewareOptions.soGracePeriod;
    soTrialPeriod        := FSharewareOptions.soTrialPeriod;
    soMaxNumRuns         := FSharewareOptions.soMaxNumRuns;
  end;
  FillChar(si,SizeOf(TSharewareInfo),0);
  FillChar(ui,SizeOf(TUserInfo),0);

  try
    { Try to read the information from the Registry }
    Result := ReadSharewareInformation(so,si,ui);

    { Check the result and raise exception if an error occurred }
    if Result <> PSERR_SUCCESS then raise EPerfectSharewareError.CreateCode(Result);

    { Evaluate the status code }
    if (so.soState and SF_DEFEATED) = SF_DEFEATED then
      FSharewareOptions.soState := FSharewareOptions.soState or SF_DEFEATED;
    if (so.soState and SF_DATEPATCHED) = SF_DATEPATCHED then
      FSharewareOptions.soState := FSharewareOptions.soState or SF_DATEPATCHED;
    if (so.soState and SF_INFOPATCHED) = SF_INFOPATCHED then
      FSharewareOptions.soState := FSharewareOptions.soState or SF_INFOPATCHED;
    if (so.soState and SF_EXPIRED) = SF_EXPIRED then
      FSharewareOptions.soState := FSharewareOptions.soState or SF_EXPIRED;

    NotifyRefresh;
  except
    { Hand on exceptions }
    raise;
  end;
end;

function TPerfectShareware.UserRegistration(const AName, ACode, APluginData: String): Boolean;
begin
  { Do not register a use when in design mode }
  Result := False;
  if (csDesigning in ComponentState) then exit;

  { Don't register twice }
  if FSharewareInfo.siRegistered then begin
    Result := True;
    exit;
  end;

  { Try to register a user and return TRUE on success }
  Result := RegisterUser(AName,StrPas(FSharewareOptions.soAppID),APluginData,ACode,FSharewareInfo,FUserInfo) = PSERR_SUCCESS;
end;


// Property methods //////////////////////////////////////////////////////////////////////////////////////////////////////////////
procedure TPerfectShareware.SetAppID(Value: String);
var
  s: String;
begin
  with FSharewareOptions do begin
    s := Trim(Value);
    if AnsiCompareText(StrPas(soAppID),s) <> 0 then begin
      if (Length(s) < 1) or (Length(s) > 128) then
        MessageDlg(Format(SInvalidValueLength,[1,128]),mtError,[mbOK],0)
      else begin
        FillChar(soAppID,SizeOf(soAppID),0);
        StrPCopy(soAppID,s);
      end;
    end;
  end;
end;

procedure TPerfectShareware.SetAutoActive(Value: Boolean);
begin
 if FAutoActive <> Value then
   FAutoActive := Value;
end;

procedure TPerfectShareware.SetEnableCheckDefeat(Value: Boolean);
begin
  with FSharewareOptions do
    if soEnableCheckDefeat <> Value then soEnableCheckDefeat := Value;
end;

procedure TPerfectShareware.SetEnableSaveOnDefeat(Value: Boolean);
begin
  with FSharewareOptions do
    if soEnableSaveOnDefeat <> Value then soEnableSaveOnDefeat := Value;
end;

procedure TPerfectShareware.SetExpirationDate(Value: TDateTime);
begin
  with FSharewareOptions do
    if soExpirationDate <> Value then soExpirationDate := Value;
end;

procedure TPerfectShareware.SetExpirationMode(Value: TExpirationMode);
begin
  with FSharewareOptions do
    if soExpirationMode <> Ord(Value) then soExpirationMode := Ord(Value);
end;

procedure TPerfectShareware.SetGracePeriod(Value: TEvaluationPeriod);
begin
  with FSharewareOptions do
    if soGracePeriod <> Value then soGracePeriod := Value;
end;

procedure TPerfectShareware.SetMaxNumRuns(Value: TRunCount);
begin
  with FSharewareOptions do
    if soMaxNumRuns <> Value then soMaxNumRuns := Value;
end;

procedure TPerfectShareware.SetRegBackupKey(Value: String);
var
  s: String;
begin
  with FSharewareOptions do begin
    s := Trim(Value);
    if AnsiCompareText(StrPas(soRegBackupKey),s) <> 0 then begin
      if (Length(s) < 1) or (Length(s) > 1024) then
        MessageDlg(Format(SInvalidValueLength,[1,1024]),mtError,[mbOK],0)
      else begin
        FillChar(soRegBackupKey,SizeOf(soRegBackupKey),0);
        StrPCopy(soRegBackupKey,s);
      end;
    end;
  end;
end;

procedure TPerfectShareware.SetRegBackupRoot(Value: TRegRootKey);
begin
  with FSharewareOptions do
    if soRegBackupRoot <> RootKeyToHKey(Value) then soRegBackupRoot := RootKeyToHKey(Value);
end;

procedure TPerfectShareware.SetRegKey(Value: String);
var
  s: String;
begin
  with FSharewareOptions do begin
    s := Trim(Value);
    if AnsiCompareText(StrPas(soRegKey),s) <> 0 then begin
      if (Length(s) < 1) or (Length(s) > 1024) then
        MessageDlg(Format(SInvalidValueLength,[1,1024]),mtError,[mbOK],0)
      else begin
        FillChar(soRegKey,SizeOf(soRegKey),0);
        StrPCopy(soRegKey,s);
      end;
    end;
  end;
end;

procedure TPerfectShareware.SetRegRoot(Value: TRegRootKey);
begin
  with FSharewareOptions do
    if soRegRoot <> RootKeyToHKey(Value) then soRegRoot := RootKeyToHKey(Value);
end;

procedure TPerfectShareware.SetTrialPeriod(Value: TEvaluationPeriod);
begin
  with FSharewareOptions do
    if soTrialPeriod <> Value then soTrialPeriod := Value;
end;

procedure TPerfectShareware.SetInitialRun(Value: TDateTime);
begin
  if FSharewareInfo.siInitialRun <> Trunc(Value) then
    FSharewareInfo.siInitialRun := Trunc(Value)
end;

procedure TPerfectShareware.SetLastRun(Value: TDateTime);
begin
  if FSharewareInfo.siLastRun <> Trunc(Value) then
    FSharewareInfo.siLastRun := Trunc(Value);
end;

procedure TPerfectShareware.SetNumRuns(Value: TRunCount);
begin
  if FSharewareInfo.siRunCount <> Value then
    FSharewareInfo.siRunCount := Value;
end;

procedure TPerfectShareware.SetRegisteredUser(Value: String);
begin
  StrPCopy(FUserInfo.uiUser, Value);
end;

procedure TPerfectShareware.SetRegistrationCode(Value: String);
begin
  StrPCopy(FUserInfo.uiCode, Value);
end;

function TPerfectShareware.GetAppID: String;
begin
  Result := StrPas(FSharewareOptions.soAppID);
end;

function TPerfectShareware.GetEnableCheckDefeat: Boolean;
begin
  Result := FSharewareOptions.soEnableCheckDefeat;
end;

function TPerfectShareware.GetExpirationDate: TDateTime;
begin
  Result := FSharewareOptions.soExpirationDate;
end;

function TPerfectShareware.GetExpirationMode: TExpirationMode;
begin
  Result := TExpirationMode(FSharewareOptions.soExpirationMode);
end;

function TPerfectShareware.GetGracePeriod: TEvaluationPeriod;
begin
  Result := FSharewareOptions.soGracePeriod;
end;

function TPerfectShareware.GetMaxNumRuns: TRunCount;
begin
  Result := FSharewareOptions.soMaxNumRuns;
end;

function TPerfectShareware.GetEnableSaveOnDefeat: Boolean;
begin
  Result := FSharewareOptions.soEnableSaveOnDefeat;
end;

function TPerfectShareware.GetRegBackupKey: String;
begin
  Result := StrPas(FSharewareOptions.soRegBackupKey);
end;

function TPerfectShareware.GetRegBackupRoot: TRegRootKey;
begin
  Result := HKeyToRootKey(FSharewareOptions.soRegBackupRoot);
end;

function TPerfectShareware.GetRegKey: String;
begin
  Result := StrPas(FSharewareOptions.soRegKey);
end;

function TPerfectShareware.GetRegRoot: TRegRootKey;
begin
  Result := HKeyToRootKey(FSharewareOptions.soRegRoot);
end;

function TPerfectShareware.GetTrialPeriod: TEvaluationPeriod;
begin
  Result := FSharewareOptions.soTrialPeriod;
end;

function TPerfectShareware.GetIsInitialRun: Boolean;
begin
  Result := (FSharewareOptions.soState and SF_INITIALRUN) = SF_INITIALRUN;
end;

function TPerfectShareware.GetIsInfoPatched: Boolean;
begin
  Result := (FSharewareOptions.soState and SF_INFOPATCHED) = SF_INFOPATCHED;
end;

function TPerfectShareware.GetIsDatePatched: Boolean;
begin
  Result := (FSharewareOptions.soState and SF_DATEPATCHED) = SF_DATEPATCHED;
end;

function TPerfectShareware.GetIsDefeated: Boolean;
begin
  Result := (FSharewareOptions.soState and SF_DEFEATED) = SF_DEFEATED;
end;

function TPerfectShareware.GetIsExpired: Boolean;
begin
  Result := (FSharewareOptions.soState and SF_EXPIRED) = SF_EXPIRED;
end;

function TPerfectShareware.GetIsRegistered: Boolean;
begin
  Result := (FSharewareOptions.soState and SF_REGISTERED) = SF_REGISTERED;
end;

function TPerfectShareware.GetInitialRun: TDateTime;
begin
  Result := FSharewareInfo.siInitialRun;
end;

function TPerfectShareware.GetLastRun: TDateTime;
begin
  Result := FSharewareInfo.siLastRun;
end;

function TPerfectShareware.GetNumRuns: TRunCount;
begin
  Result := FSharewareInfo.siRunCount;
end;

function TPerfectShareware.GetRemainingTrialPeriod: Integer;
var
  i: Integer;
begin
  i := FSharewareOptions.soTrialPeriod-(Trunc(Now)-Trunc(GetInitialRun));
  if i < 0 then Result := -1 else Result := i;
end;

function TPerfectShareware.GetRemainingGracePeriod: Integer;
var
  i: Integer;
begin
  if GetRemainingTrialPeriod > 0 then
    Result := FSharewareOptions.soGracePeriod
  else begin
    i := FSharewareOptions.soGracePeriod-(Trunc(Now)-Trunc(GetInitialRun))+FSharewareOptions.soTrialPeriod+1;
    if i < 0 then Result := -1 else Result := i;
  end;
end;

function TPerfectShareware.GetRemainingRuns: Integer;
var
  i: Integer;
begin
  i := FSharewareOptions.soMaxNumRuns-GetNumRuns;
  if i < 0 then Result := 0 else Result := i;
end;

function TPerfectShareware.GetRegisteredUser: String;
begin
  if GetIsRegistered then
    Result := StrPas(FUserInfo.uiUser)
  else
    Result := '';
end;

function TPerfectShareware.GetRegistrationCode: String;
begin
  if GetIsRegistered then
    Result := StrPas(FUserInfo.uiCode)
  else
    Result := '';
end;


// Tool helpers //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function TPerfectShareware.RootKeyToHKey(AKey: TRegRootKey): HKEY;
begin
  Result := HKEY_CLASSES_ROOT;
  case AKey of
    hkCurrentUser : Result := HKEY_CURRENT_USER;
    hkLocalMachine: Result := HKEY_LOCAL_MACHINE;
    hkUsers       : Result := HKEY_USERS;
    hkDynData     : Result := HKEY_DYN_DATA;
  end;
end;

function TPerfectShareware.HKeyToRootKey(AKey: HKEY): TRegRootKey;
begin
  Result := hkClassesRoot;
  case AKey of
    HKEY_CURRENT_USER : Result := hkCurrentUser;
    HKEY_LOCAL_MACHINE: Result := hkLocalMachine;
    HKEY_USERS        : Result := hkUsers;
    HKEY_DYN_DATA     : Result := hkDynData;
  end;
end;

procedure TPerfectShareware.NotifyInfoPatched;
begin
  if Assigned(FOnInfoPatched) then FOnInfoPatched(Self);
end;

procedure TPerfectShareware.NotifyDatePatched;
begin
  if Assigned(FOnDatePatched) then FOnDatePatched(Self);
end;

procedure TPerfectShareware.NotifyDefeated;
begin
  if Assigned(FOnDefeated) then FOnDefeated(Self);
end;

procedure TPerfectShareware.NotifyExpired;
begin
  if Assigned(FOnExpired) then FOnExpired(Self);
end;

procedure TPerfectShareware.NotifyNagScreen;
begin
  if Assigned(FOnNagScreen) then FOnNagScreen(Self);
end;

procedure TPerfectShareware.NotifyInitialRun;
begin
  if Assigned(FOnInitialRun) then FOnInitialRun(Self);
end;

procedure TPerfectShareware.NotifyRefresh;
begin
  if Assigned(FOnRefresh) then FOnRefresh(Self);
end;

end.
