unit ps40PropEdits;

interface

uses Forms, DsgnIntf;

type
  { TPerfectSharewareEditor }
  TPerfectSharewareEditor = class(TComponentEditor)
  public
    function  GetVerbCount: Integer; override;
    function  GetVerb(Index: Integer): String; override;
    procedure ExecuteVerb(Index: Integer); override;
  end;

  { TExpirationDateProperty }
  TExpirationDateProperty = class(TPropertyEditor)
  public
    function  GetAttributes: TPropertyAttributes; override;
    function  GetValue: String; override;
    procedure SetValue(const Value: String); override;
    procedure Edit; override;
  end;


procedure RegisterPropertyEditors;

implementation

uses Windows, ps40, SysUtils, Controls, psVersionDlg, psPickDate;

const
  SClearRegInfoWarning = 'You are about to remove information from the Windows Registry!'#13#13#10'Before you continue, it is '+
                         'strongly recommended that you create a backup of the Registry using REGEDIT.EXE. Using this command '+
                         'might cause the Registry to be damaged.'#13#13#10'Do you wish to continue anyway?';

procedure RegisterPropertyEditors;
begin
  RegisterComponentEditor(TPerfectShareware,TPerfectSharewareEditor);
  RegisterPropertyEditor(TypeInfo(TDateTime),TPerfectShareware,'',TExpirationDateProperty);
end;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function TPerfectSharewareEditor.GetVerbCount: Integer;
begin
  { There are currently two verbs }
  Result := 2;
end;

function TPerfectSharewareEditor.GetVerb(Index: Integer): String;
begin
  { Return the menu item strings for the verbs }
  case Index of
    0 : Result := 'About &PerfectShareware...';
    1 : Result := 'Clear Registry &Keys...';
  end;
end;

procedure TPerfectSharewareEditor.ExecuteVerb(Index: Integer);
begin
  { Execute the verb associated with the index }
  case Index of
    0 : ShowComponentVersion;
    1 : begin
          if Application.MessageBox(PChar(SClearRegInfoWarning),'Warning',MB_ICONWARNING or MB_YESNO) = IDYES then
            (Component as TPerfectShareware).ClearSecretInfo;
        end;
  end;
end;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function TExpirationDateProperty.GetAttributes: TPropertyAttributes;
begin
  { Dialog is REQUIRED, the property is read only }
  Result := [paDialog, paReadOnly];
end;

function TExpirationDateProperty.GetValue: String;
var
  ADate: TDateTime;
begin
  { Return a string displayable in the Object Inspector }
  ADate := GetFloatValue;
  if ADate <> 0.0 then
    Result := DateToStr(ADate)
  else
    Result := '(Not specified)';
end;

procedure TExpirationDateProperty.SetValue(const Value: String);
var
  ADate: TDateTime;
begin
  { Create the value from the string in the Object Inspector }
  if (Value = '') or (Value = '(Not specified)') then
    ADate := 0.0
  else
    ADate := StrToDate(Value);
  SetFloatValue(ADate);
end;

procedure TExpirationDateProperty.Edit;
var
  ADay,AMonth,AYear: Word;
  ADate            : TDateTime;
  DateDlg          : TPickDateDlg;
begin
  { Set the property by displaying the dialog }
  DateDlg := TPickDateDlg.Create(Application);
  try
    { Decode the date (current date on error) }
    try
      DecodeDate(StrToDate(GetValue),AYear,AMonth,ADay);
    except
      DecodeDate(Date,AYear,AMonth,ADay);
    end;

    { Show the dialog and retrieve the selected date when OK was clicked }
    if DateDlg.Execute(ADay,AMonth,AYear) then begin
      ADate := EncodeDate(AYear,AMonth,ADay);
      SetValue(DateToStr(ADate));
    end;
  finally
    DateDlg.Free;
  end;
end;


end.
