{ ********************************************************************************************************** }
{ ps40import.pas                                            Copyright  1998-99 DithoSoft Software Solutions }
{ Version 4.0                                                                                                }
{ ********************************************************************************************************** }
{ This unit contains constants, types and function declarations required to use the PerfectShareware         }
{ Library. This unit is used by the TPerfectShareware component, but you can use it as it is and interact    }
{ with the PerfectShareware Library "manually".                                                              }
{ If you would like to use the PerfectShareware Library with a programming language other than Delphi,       }
{ translate this unit into your programming language and refer to the DLL-only sample application for how to }
{ directly access the PerfectShareware Library.                                                              }
{ ********************************************************************************************************** }
unit ps40import;

interface

uses Windows;

const
  { Error return codes }
  PSERR_SUCCESS              = 0;     // No error occurred
  PSERR_REGISTRYREAD         = 1;     // An error occurred while reading from the Registry
  PSERR_REGISTRYWRITE        = 2;     // An error occurred while writing to the Registry
  PSERR_NOKEY                = 3;     // The specified Registry key was not found
  PSERR_NOINFORMATION        = 4;     // No shareware information was found at the specified position
  PSERR_SAVEDENIED           = 5;     // The shareware mechanism was defeated -> did not write
  PSERR_INVALIDAPPID         = 6;     // You did not specify an appliction ID
  PSERR_INVALIDREGROOT       = 7;     // You did not specify a root key
  PSERR_INVALIDREGBACKUPROOT = 8;     // You did not specify a backup root key
  PSERR_INVALIDREGKEY        = 9;     // You did not specify the Registry key
  PSERR_INVALIDREGBACKUPKEY  = 10;    // You did not specify the Registry backup key
  PSERR_REGISTRATIONFAILED   = 11;    // The registration failed
  PSERR_LIBRARYIMPORT        = 12;    // The PerfectShareware Library could not be imported

  { Shareware state return flags }
  SF_NOSTATE                = 0;      // The state could not be determined or is normal -> Nag screen
  SF_INITIALRUN             = 1;      // The application was run for the first time
  SF_DEFEATED               = 2;      // The user tried to defeat the shareware mechanism
  SF_DATEPATCHED            = 4;      // The user turned back the clock to defeat the shareware mechanism
  SF_INFOPATCHED            = 8;      // The user patched one of the Registry sets
  SF_OVERTRIALPERIOD        = 16;     // The trial period is over, but the grace period is left
  SF_EXPIRED                = 32;     // The application expired
  SF_REGISTERED             = 64;     // The application is registered

  { Expiration mode state flags }
  EM_DONTEXPIRE             = 0;      // The application should never expire
  EM_TRIALPERIOD            = 1;      // The application should expire after a trial/grace period
  EM_RUNCOUNT               = 2;      // The application should expire after a certain number of runs
  EM_COMBINED               = 3;      // The application should expire after trial/grace or number of runs
  EM_EXPIRATIONDATE         = 4;      // The application should expire on a specified day

type
  { TSharewareInfo - stored in the registry for information on status }
  PSharewareInfo = ^TSharewareInfo;
  TSharewareInfo = packed record
    siInitialRun         : TDateTime;         // The date when the application was run for the first time
    siLastRun            : TDateTime;         // The date when the application was run last time
    siRunCount           : Integer;           // The total number of runs
    siRegistered         : Boolean;           // The registration state (True when registered)
  end;

  { TUserInfo - stored in the registry for information on user }
  PUserInfo = ^TUserInfo;
  TUserInfo = packed record
    uiUser               : Array[0..255] of Char;   // The name of the registered user
    uiCode               : Array[0..31] of Char;    // The registration code the user used to register
  end;

  { TSharewareOptions - used to determine the shareware behaviour }
  PSharewareOptions = ^TSharewareOptions;
  TSharewareOptions = packed record
    soAppID              : Array[0..128] of Char;  // The unique application ID
    soEnableCheckDefeat  : Boolean;                // True when library should check for defeats
    soEnableSaveOnDefeat : Boolean;                // True when library should save even in case of defeats
    soRegRoot            : HKEY;                   // Root key for original Registry information
    soRegBackupRoot      : HKEY;                   // Root key for Registry backup information
    soRegKey             : Array[0..1024] of Char; // Key for orginal Registry info (AppID will be appended)
    soRegBackupKey       : Array[0..1024] of Char; // Key for Registry backup info (AppID will be appended)
    soExpirationMode     : Integer;                // Type of shareware application (see above for flags)
    soExpirationDate     : TDateTime;              // Date of expiration
    soGracePeriod        : Integer;                // Number of days of the grace period
    soTrialPeriod        : Integer;                // Number of days of the trial period
    soMaxNumRuns         : Integer;                // Maximum number of runs
    soState              : LongInt;                // State return flag (DO NOT MODIFY!!)
  end;


{ Get information about the library }
function GetDLLName: String;
function GetDLLVersion: String;

{ Read/Write information to the registry }
function ReadSharewareInformation(var Options: TSharewareOptions; var si: TSharewareInfo; var ui: TUserInfo): DWORD;
function WriteSharewareInformation(var Options: TSharewareOptions; var si: TSharewareInfo; var ui: TUserInfo): DWORD;

{ Create a registration codes for a user/Register the user }
function CreateRegistrationCode(const AUserName, AAppID, APluginData: String): String;
function RemoveSharewareInformation(var Options: TSharewareOptions): DWORD;

{ Remove current shareware information }
function RegisterUser(const AUserName, AAppID, APluginData, ARegCode: String; var mi: TSharewareInfo; var ui: TUserInfo): DWORD;

implementation

uses SysUtils;

{ ******************************************************************************************************************************
  The following functions are exported by the PerfectShareware Library. This is the original Object Pascal syntax. The functions
  are loaded dynamically to allow the use of simpler wrapper routines.

  function GetDLLName: PChar; stdcall;
  function GetDLLVersion: PChar; stdcall;
  function ReadSharewareInformation(Options: PSharewareOptions; si: PSharewareInfo; ui: PUserInfo): DWORD; stdcall;
  function WriteSharewareInformation(Options: PSharewareOptions; si: PSharewareInfo; ui: PUserInfo): DWORD; stdcall;
  function CreateRegistrationCode(AUserName,AAppID,APluginData: PChar): PChar; stdcall;
  function RemoveSharewareInformation(Options: PSharewareOptions): DWORD; stdcall;
  function RegisterUser(AUserName, AAppKey, APluginData, ARegCode: PChar; mi: PSharewareInfo; ui: PUserInfo): DWORD; stdcall;
  ****************************************************************************************************************************** }

type
  TFNGetDllName = function: PChar; stdcall;
  TFNGetDLLVersion = function: PChar; stdcall;
  TFNReadSharewareInformation = function(Options: PSharewareOptions; si: PSharewareInfo; ui: PUserInfo): DWORD; stdcall;
  TFNWriteSharewareInformation = function(Options: PSharewareOptions; si: PSharewareInfo; ui: PUserInfo): DWORD; stdcall;
  TFNCreateRegistrationCode = function(AUserName, AAppID, APluginData: PChar): PChar; stdcall;
  TFNRemoveSharewareInformation = function(Options: PSharewareOptions): DWORD; stdcall;
  TFNRegisterUser = function(AUserName, AAppKey, APluginData, ARegCode: PChar; mi: PSharewareInfo; ui: PUserInfo): DWORD; stdcall;

var
  FGetDLLName                  : TFNGetDLLName = nil;
  FGetDLLVersion               : TFNGetDLLVersion = nil;
  FReadSharewareInformation    : TFNReadSharewareInformation = nil;
  FWriteSharewareInformation   : TFNWriteSharewareInformation = nil;
  FCreateRegistrationCode      : TFNCreateRegistrationCode = nil;
  FRemoveSharewareInformation  : TFNRemoveSharewareInformation = nil;
  FRegisterUser                : TFNRegisterUser = nil;
  FDLLHandle                   : THandle = 0;


function GetDLLName: String;
begin
  if @FGetDLLName <> nil then
    Result := StrPas(FGetDLLName)
  else
    Result := '';
end;

function GetDLLVersion: String;
begin
  if @FGetDLLVersion <> nil then
    Result := StrPas(FGetDLLVersion)
  else
    Result := '';
end;

function ReadSharewareInformation(var Options: TSharewareOptions; var si: TSharewareInfo; var ui: TUserInfo): DWORD;
begin
  if @FReadSharewareInformation <> nil then
    Result := FReadSharewareInformation(@Options,@si,@ui)
  else
    Result := PSERR_LIBRARYIMPORT;
end;

function WriteSharewareInformation(var Options: TSharewareOptions; var si: TSharewareInfo; var ui: TUserInfo): DWORD;
begin
  if @FWriteSharewareInformation <> nil then
    Result := FWriteSharewareInformation(@Options,@si,@ui)
  else
    Result := PSERR_LIBRARYIMPORT;
end;

function CreateRegistrationCode(const AUserName, AAppID, APluginData: String): String;
begin
  if @FCreateRegistrationCode <> nil then
    Result := StrPas(FCreateRegistrationCode(PChar(AUserName),PChar(AAppID),PChar(APluginData)))
  else
    Result := '';
end;

function RemoveSharewareInformation(var Options: TSharewareOptions): DWORD;
begin
  if @FRemoveSharewareInformation <> nil then
    Result := FRemoveSharewareInformation(@Options)
  else
    Result := PSERR_LIBRARYIMPORT;
end;

function RegisterUser(const AUserName, AAppID, APluginData, ARegCode: String; var mi: TSharewareInfo; var ui: TUserInfo): DWORD;
begin
  if @FRegisterUser <> nil then
    Result := FRegisterUser(PChar(AUserName),PChar(AAppID),PChar(APluginData),PChar(ARegCode),@mi,@ui)
  else
    Result := PSERR_LIBRARYIMPORT;
end;

initialization
  FDLLHandle := LoadLibrary('ps40.dll');
  if FDLLHandle <> 0 then begin
    @FGetDLLName                 := GetProcAddress(FDLLHandle,'GetDLLName');
    @FGetDLLVersion              := GetProcAddress(FDLLHandle,'GetDLLVersion');
    @FReadSharewareInformation   := GetProcAddress(FDLLHandle,'ReadSharewareInformation');
    @FWriteSharewareInformation  := GetProcAddress(FDLLHandle,'WriteSharewareInformation');
    @FCreateRegistrationCode     := GetProcAddress(FDLLHandle,'CreateRegistrationCode');
    @FRemoveSharewareInformation := GetProcAddress(FDLLHandle,'RemoveSharewareInformation');
    @FRegisterUser               := GetProcAddress(FDLLHandle,'RegisterUser');
  end;

finalization
  if FDLLHandle <> 0 then FreeLibrary(FDLLHandle);

end.
